/*
 * Copyright E. Bigeon (2015)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a swt window for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc-swt:fr.bigeon.gclc.swt.SWTConsole.java
 * Created on: Apr 18, 2015
 */
package fr.bigeon.gclc.swt;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.manager.PipedConsoleInput;
import fr.bigeon.gclc.manager.PipedConsoleOutput;
import fr.bigeon.gclc.tools.AOutputForwardRunnable;

/** A SWT component to connect to gclc {@link ConsoleApplication}
 * <p>
 *
 * @author Emmanuel Bigeon */
public class SWTConsoleView extends Composite implements ConsoleDelayIO {
    /** The local implementation of the forwarding runnable
     *
     * @author Emmanuel Bigeon */
    private final class ToSWTConsoleForwardRunnable
            extends AOutputForwardRunnable {
        /** The running status */
        private boolean running = true;

        /** @param manager the manager */
        public ToSWTConsoleForwardRunnable(final PipedConsoleOutput manager) {
            super(manager);
        }

        @Override
        protected void forwardLine(final String m) {
            appendConsoleOutput(m);
        }

        @Override
        protected boolean isRunning() {
            return running && !isDisposed();
        }

        /** @param running the running to set */
        public void setRunning(final boolean running) {
            this.running = running;
        }
    }

    /** The class logger */
    private static final Logger LOGGER = Logger
            .getLogger(SWTConsoleView.class.getName());
    /** The console output text field */
    private final Text consoleOutput;
    /** The console input text field */
    private final Text consoleInput;
    /** The actual manager */
    private PipedConsoleOutput manager;
    private PipedConsoleInput input;
    /** The forwarding runnable */
    private ToSWTConsoleForwardRunnable forward;

    /** Create the composite.
     *
     * @param parent the prent composite
     * @param style the composite style */
    public SWTConsoleView(final Composite parent, final int style) {
        super(parent, style);

        setLayout(new GridLayout(1, false));

        consoleOutput = new Text(this, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP |
                                       SWT.V_SCROLL | SWT.MULTI);
        consoleOutput.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true,
                1, 1));
        consoleOutput.setRedraw(true);

        consoleInput = new Text(this, SWT.BORDER);
        consoleInput.setLayoutData(
                new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        consoleInput.addKeyListener(new HistoryTextKeyListener(this));
    }

    /** @param next the next message */
    protected void appendConsoleOutput(final String next) {
        Display.getDefault().syncExec(new Runnable() {
            @SuppressWarnings("synthetic-access")
            @Override
            public void run() {
                consoleOutput.append(System.lineSeparator() + next);
            }
        });
    }

    @Override
    protected void checkSubclass() {
        // Disable the check that prevents subclassing of SWT components
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.swt.ConsoleDelayIO#getInput() */
    @Override
    public String getInput() {
        return consoleInput.getText();
    }

    /* (non-Javadoc)
     * @see org.eclipse.swt.widgets.Composite#setFocus() */
    @Override
    public boolean setFocus() {
        return consoleInput.setFocus();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.swt.ConsoleDelayIO#setInput(java.lang.String) */
    @Override
    public void setInput(final String input) {
        consoleInput.setText(input);
        consoleInput.setSelection(input.length());
    }

    /** @param manager the manager to set */
    public void setManager(final PipedConsoleOutput manager,
                           final PipedConsoleInput input) {
        this.manager = manager;
        this.input = input;
        if (forward != null) {
            forward.setRunning(false);
        }
        forward = new ToSWTConsoleForwardRunnable(manager);
        final Thread th = new Thread(forward, "gclcToSWT"); //$NON-NLS-1$
        th.start();
    }

    /** @param string the text */
    public void setText(final String string) {
        consoleInput.setText(string);
    }

    /**
     *
     */
    @Override
    public void validateInput() {
        try {
            input.type(getInput());
        } catch (final IOException e) {
            LOGGER.log(Level.SEVERE, "Unable to input value to console", e); //$NON-NLS-1$
        }
    }
}
